#include "ring_buffer.h"


void ring_buffer_init(ring_buffer_t *buffer)
{
    buffer->read_pointer = 0;
    ring_buffer_clear(buffer);
}


void ring_buffer_clear(ring_buffer_t *buffer)
{
    buffer->write_pointer = buffer->read_pointer;
    buffer->data_length = 0;
}


bool ring_buffer_is_empty(ring_buffer_t const *buffer)
{
    return (buffer->data_length == 0);
}


bool ring_buffer_is_full(ring_buffer_t const *buffer)
{
    return (buffer->data_length == RING_BUFFER_CAPACITY);
}


static void ring_buffer_advance_pointer(uint8_t *pointer)
{
    ++(*pointer);

    if (*pointer >= RING_BUFFER_CAPACITY)
    {
        *pointer = 0;
    }
}


void ring_buffer_push(ring_buffer_t *buffer, uint8_t byte)
{
    buffer->storage[buffer->write_pointer] = byte;
    ring_buffer_advance_pointer(&(buffer->write_pointer));
    ++buffer->data_length;
}


uint8_t ring_buffer_pop(ring_buffer_t *buffer)
{
    uint8_t result = buffer->storage[buffer->read_pointer];
    ring_buffer_advance_pointer(&(buffer->read_pointer));
    --buffer->data_length;
    return result;
}